@@ -121,6 +121,7 @@ i830_pixmap_tiled(PixmapPtr pPixmap)
     return FALSE;
 }
 
+#if EXA_VERSION_MINOR >= 2
 static Bool
 i830_exa_pixmap_is_offscreen(PixmapPtr pPixmap)
 {
@@ -136,6 +137,7 @@ i830_exa_pixmap_is_offscreen(PixmapPtr pPixmap)
 	return FALSE;
     }
 }
+#endif
 
 /**
  * I830EXASync - wait for a command to finish
@@ -170,6 +172,8 @@ I830EXAPrepareSolid(PixmapPtr pPixmap, int alu, Pixel planemask, Pixel fg)
     if (pPixmap->drawable.bitsPerPixel == 24)
 	I830FALLBACK("solid 24bpp unsupported!\n");
 
+    i830_exa_check_pitch_2d(pPixmap);
+
     offset = exaGetPixmapOffset(pPixmap);
     pitch = exaGetPixmapPitch(pPixmap);
 
@@ -255,6 +259,9 @@ I830EXAPrepareCopy(PixmapPtr pSrcPixmap, PixmapPtr pDstPixmap, int xdir,
     if (!EXA_PM_IS_SOLID(&pSrcPixmap->drawable, planemask))
 	I830FALLBACK("planemask is not solid");
 
+    i830_exa_check_pitch_2d(pSrcPixmap);
+    i830_exa_check_pitch_2d(pDstPixmap);
+
     pI830->pSrcPixmap = pSrcPixmap;
 
     pI830->BR[13] = I830CopyROP[alu] << 16;
@@ -339,12 +346,31 @@ I830EXADoneCopy(PixmapPtr pDstPixmap)
 #define xFixedToFloat(val) \
 	((float)xFixedToInt(val) + ((float)xFixedFrac(val) / 65536.0))
 
+static Bool
+_i830_transform_point (PictTransformPtr transform,
+		       float		x,
+		       float		y,
+		       float		result[3])
+{
+    int		    j;
+
+    for (j = 0; j < 3; j++)
+    {
+	result[j] = (xFixedToFloat (transform->matrix[j][0]) * x +
+		     xFixedToFloat (transform->matrix[j][1]) * y +
+		     xFixedToFloat (transform->matrix[j][2]));
+    }
+    if (!result[2])
+	return FALSE;
+    return TRUE;
+}
+
 /**
  * Returns the floating-point coordinates transformed by the given transform.
  *
  * transform may be null.
  */
-void
+Bool
 i830_get_transformed_coordinates(int x, int y, PictTransformPtr transform,
 				 float *x_out, float *y_out)
 {
@@ -352,15 +378,14 @@ i830_get_transformed_coordinates(int x, int y, PictTransformPtr transform,
 	*x_out = x;
 	*y_out = y;
     } else {
-	PictVector v;
+	float	result[3];
 
-        v.vector[0] = IntToxFixed(x);
-        v.vector[1] = IntToxFixed(y);
-        v.vector[2] = xFixed1;
-        PictureTransformPoint(transform, &v);
-	*x_out = xFixedToFloat(v.vector[0]);
-	*y_out = xFixedToFloat(v.vector[1]);
+	if (!_i830_transform_point (transform, (float) x, (float) y, result))
+	    return FALSE;
+	*x_out = result[0] / result[2];
+	*y_out = result[1] / result[2];
     }
+    return TRUE;
 }
 
 /**
@@ -368,7 +393,7 @@ i830_get_transformed_coordinates(int x, int y, PictTransformPtr transform,
  *
  * transform may be null.
  */
-void
+Bool
 i830_get_transformed_coordinates_3d(int x, int y, PictTransformPtr transform,
 				    float *x_out, float *y_out, float *w_out)
 {
@@ -377,16 +402,15 @@ i830_get_transformed_coordinates_3d(int x, int y, PictTransformPtr transform,
 	*y_out = y;
 	*w_out = 1;
     } else {
-	PictVector v;
+	float    result[3];
 
-        v.vector[0] = IntToxFixed(x);
-        v.vector[1] = IntToxFixed(y);
-        v.vector[2] = xFixed1;
-        PictureTransformPoint3d(transform, &v);
-	*x_out = xFixedToFloat(v.vector[0]);
-	*y_out = xFixedToFloat(v.vector[1]);
-	*w_out = xFixedToFloat(v.vector[2]);
+	if (!_i830_transform_point (transform, (float) x, (float) y, result))
+	    return FALSE;
+	*x_out = result[0];
+	*y_out = result[1];
+	*w_out = result[2];
     }
+    return TRUE;
 }
 
 /**
